@@ -3,6 +3,7 @@ use super::{
     headers::{DidcommHeader, JwmHeader},
     mediated::Mediated,
     prior_claims::PriorClaims,
+    Attachment,
 };
 #[cfg(feature = "raw-crypto")]
 use crate::crypto::{CryptoAlgorithm, Cypher, SignatureAlgorithm, Signer};
@@ -31,13 +32,15 @@ pub struct Message {
     pub jwm_header: JwmHeader,
     /// DIDComm headers part, sent as part of encrypted message in JWE.
     #[serde(flatten)]
-    didcomm_header: DidcommHeader,
+    pub(crate) didcomm_header: DidcommHeader,
     #[serde(skip_serializing_if = "Option::is_none")]
     pub(crate) recepients: Option<Vec<Recepient>>,
     /// Message payload, which can be basically anything (JSON, text, file, etc.) represented
     ///     as base64url String of raw bytes of data.
     /// No direct access for encode/decode purposes! Use `get_body()` / `set_body()` methods instead.
     body: String,
+    #[serde(skip_serializing_if = "Vec::is_empty", default)]
+    pub(crate) attachments: Vec<Attachment>,
 }
 
 impl Message {
@@ -50,8 +53,34 @@ impl Message {
             didcomm_header: DidcommHeader::new(),
             recepients: None,
             body: String::default(),
+            attachments: Vec::new(),
         }
     }
+    /// Shortcut to `DidcommHeader::get_message_uri`
+    ///
+    pub fn get_message_uri(&self) -> String {
+        self.didcomm_header.get_message_uri()
+    }
+    /// Sets `thid` and `pthid` same as those in `replying_to`
+    /// Shortcut to `DidcommHeader::reply_to` method
+    ///
+    /// # Parameters
+    ///
+    /// * `replying_to` - ref to message we're replying to
+    pub fn reply_to(mut self, replying_to: &Self) -> Self {
+        self.didcomm_header.reply_to(&replying_to.didcomm_header);
+        self
+    }
+    /// Sets `pthid` to the one specified in `parent`'s `thid`
+    ///
+    /// # Parameters
+    ///
+    /// * `parent` - ref to a parent threaded `Message`
+    ///
+    pub fn with_parent(mut self, parent: &Self) -> Self {
+        self.didcomm_header.pthid = parent.didcomm_header.thid.clone();
+        self
+    }
     /// Setter of `from` header
     /// Helper method.
     ///
@@ -79,8 +108,18 @@ impl Message {
     /// Setter of `m_type` @type header
     /// Helper method.
     ///
-    pub fn m_type(mut self, m_type: MessageType) -> Self {
-        self.didcomm_header.m_type = m_type;
+    pub fn m_type(mut self, m_type: &str) -> Self {
+        self.didcomm_header.m_type = m_type.into();
+        self
+    }
+    /// Setter of `typ` header property.
+    ///
+    /// # Parameters
+    ///
+    /// * `typ` - `MessageType` to be set for `typ` property
+    ///
+    pub fn typ(mut self, typ: MessageType) -> Self {
+        self.jwm_header.typ = typ;
         self
     }
     /// Getter of the `body` as ref of bytes slice.
@@ -154,13 +193,18 @@ impl Message {
     /// Adds (or updates) custom unique header key-value pair to the header.
     /// This portion of header is not sent as JOSE header.
     ///
-    pub fn add_header_field(mut self, key: String, value: String) -> Self {
+    pub fn add_header_field(mut self, key: &str, value: &str) -> Self {
         if key.len() == 0 {
             return self;
         }
-        self.didcomm_header.other.insert(key, value);
+        self.didcomm_header.other.insert(key.into(), value.into());
         self
     }
+    /// Gets `Iterator` over key-value pairs of application level headers
+    ///
+    pub fn get_application_params(&self) -> impl Iterator<Item = (&String, &String)> {
+        self.didcomm_header.other.iter()
+    }
     /// Creates set of Jwm related headers for the JWE
     /// Modifies JWM related header portion to match
     ///     encryption implementation and leaves other
@@ -306,7 +350,7 @@ impl Message {
             .as_jws(&signing_algorithm)
             .sign(signing_algorithm.signer(), sk)?;
         to.body = encode(&signed.as_bytes());
-        return to.m_type(MessageType::DidcommJws).seal(ek);
+        return to.typ(MessageType::DidcommJws).seal(ek);
     }
     // #[cfg(feature = "resolve")]
     // pub fn seal_signed(
@@ -342,7 +386,7 @@ impl Message {
             .to(&[mediator_did])
             .from(&from)
             .as_jwe(&alg)
-            .m_type(MessageType::DidcommForward)
+            .typ(MessageType::DidcommForward)
             .set_body(serde_json::to_string(&body)?.as_bytes())
             .seal(ek)
     }
@@ -446,6 +490,7 @@ impl Message {
     //                 } else {
     //                     if let Ok(mediated) = serde_json::from_slice::<Mediated>(&decode(&m.body)?) {
     //                         Ok(Message {
+
     //                             body: encode(&mediated.payload),
     //                             ..m
     //                         })
@@ -503,7 +548,7 @@ impl Message {
                             shared.as_bytes(),
                         )?;
                     }
-                    if &m.didcomm_header.m_type == &MessageType::DidcommJws {
+                    if &m.jwm_header.typ == &MessageType::DidcommJws {
                         if m.jwm_header.alg.is_none() {
                             return Err(Error::JweParseError);
                         }
